@@ -13,7 +13,7 @@ from s3_data_packer.constants import default_file_limit_gigabytes
 from s3_data_packer.helpers import get_file_format
 from s3_data_packer.s3_output_store import S3OutputStore
 from s3_data_packer.s3_table_store import S3TableStore
-from typing import List, Tuple, Union
+from typing import List, Tuple, Union, Iterable
 
 
 class S3DataPacker:
@@ -30,6 +30,7 @@ class S3DataPacker:
         output_partition: dict = None,
         input_partition_name: str = None,
         file_limit_gigabytes: int = default_file_limit_gigabytes,
+        read_chunksize: Union[int, str] = None,
     ):
 
         # set the blank table_name property. Has to be the _ one as it depends on itself
@@ -79,6 +80,8 @@ class S3DataPacker:
         if self.metadata is not None:
             self.metadata.set_col_type_category_from_types()
 
+        self.read_chunksize = read_chunksize
+
     def _get_meta(self, ext: str = None) -> Union[Metadata, None]:
         meta = (
             self.metadata
@@ -94,7 +97,7 @@ class S3DataPacker:
         ) as t:
             self.table_nrows = df.shape[0]
             writer.write(df, t.name)
-            self.file_size_on_disk = os.path.getsize(t.name) / (10 ** 9)
+            self.file_size_on_disk = os.path.getsize(t.name) / (10**9)
 
     def _get_chunk_increments(self) -> Tuple[List[int], List[int]]:
         increment_size = max(
@@ -146,30 +149,68 @@ class S3DataPacker:
     def _read_file(self, fp: str, ext: str = None) -> DataFrame:
         meta = self._get_meta(ext)
         df = reader.read(fp)
-        df = cast_pandas_table_to_schema(
-            df, meta, ignore_columns=meta.partitions
-        ) if meta else df
+        df = (
+            cast_pandas_table_to_schema(df, meta, ignore_columns=meta.partitions)
+            if meta
+            else df
+        )
         return df
 
     def pack_data(self):
         # any data to even add?
         if not self.input_store.get_files_from_table_log():
             return
+        # Will the data be read in chunks?
+        if self.read_chunksize is not None:
+            return self._pack_chunked_data()
         # collate all the data from s3
         total_df = self._append_files()
+        self._write_df(total_df)
+
+    def _write_df(self, df):
         # get the size on disk
-        self._set_file_size_on_disk(total_df)
+        self._set_file_size_on_disk(df)
         # get the indexes to write to
         start_vals, end_vals = self._get_chunk_increments()
         for i in range(len(start_vals)):
             # break up the df
-            df = total_df[start_vals[i] : end_vals[i]]
+            df_part = df[start_vals[i] : end_vals[i]]
             out_path = self.output_store._get_filename(full_path=True)
             # output the pandas df
-            writer.write(df, out_path, metadata=self.metadata)
+            writer.write(df_part, out_path, metadata=self.metadata)
             # increment the filenumber
             self.output_store.filenum += 1
 
+    def _read_chunked_file(self, fp: str, ext: str = None) -> Iterable[DataFrame]:
+        file_format = self.input_store.table_extension if ext is None else ext
+        meta = self._get_meta(ext)
+
+        for df in reader.read(
+            fp, file_format=file_format, chunksize=self.read_chunksize
+        ):
+            df = (
+                cast_pandas_table_to_schema(df, meta, ignore_columns=meta.partitions)
+                if meta
+                else df
+            )
+            yield df
+
+    def _get_dataframes(self) -> Iterable[DataFrame]:
+        # from a list of input files get chunked Dataframes
+        for f in self.input_store.get_files_from_table_log(full_path=True):
+            yield from self._read_chunked_file(f)
+
+    def _pack_chunked_data(self):
+        for df in self._get_dataframes():
+            if self.output_store._should_append_data():
+                existing_df = self._get_latest_file()
+                df = concat([existing_df, df])
+            else:
+                self.output_store.filenum += 1
+            self._write_df(df)
+            # Reset the output store for the next chunked dataframe
+            self.output_store._reset()
+
     @property
     def table_name(self):
         return self._table_name
